#!/usr/bin/env python3
import argparse
import requests

def read_config(file_path):
    try:
        with open(file_path, 'r') as file:
            for line in file:
                if line.startswith("Edition"):
                    # Split the line into words and skip the first word ("Edition")
                    words_after_edition = line.split()[1:]
                    print("Words after 'Edition':", words_after_edition)
                    return words_after_edition  # Return the list of words
            print("No line starting with 'Edition' was found.")
    except FileNotFoundError:
        print(f"Error: The file '{file_path}' does not exist.")
        return []
    except Exception as e:
        print(f"An error occurred: {e}")
        return []



def download_file(url, output_path):
    try:
        # Send a GET request to the URL
        response = requests.get(url, stream=True)
        response.raise_for_status()  # Raise an error for bad responses (4xx, 5xx)

        # Write the file to the specified path
        with open(output_path, 'wb') as file:
            for chunk in response.iter_content(chunk_size=8192):
                file.write(chunk)
        print(f"File downloaded successfully to {output_path}")
    except requests.exceptions.RequestException as e:
        print(f"An error occurred: {e}")

def main():
    parser = argparse.ArgumentParser(description="Read, parse and download db's as mentioned in the config file.")
    parser.add_argument('-f', '--file', default= "/etc/GeoIP.conf", type=str, help='Config file (default "/etc/GeoIP.conf")')
    parser.add_argument('-d', '--dbdir', default= "/var/lib/GeoIP/", type=str, help='Config file (default "/var/lib/GeoIP/")')
    args = parser.parse_args()
    
    requested_db_list = read_config(args.file)
    if len(requested_db_list) == 0:
        exit
    for db in requested_db_list:
        if db == "GeoLite2-Country":
            print("download country")
            download_file("https://cdn.jsdelivr.net/npm/@ip-location-db/geolite2-city-mmdb/geolite2-city-ipv4.mmdb", args.dbdir+"GeoLite2-Country.mmdb")
        elif db == "GeoLite2-City":
            print("download city")
            download_file("https://cdn.jsdelivr.net/npm/@ip-location-db/geolite2-city-mmdb/geolite2-city-ipv4.mmdb", args.dbdir+"GeoLite2-City.mmdb")
        elif db == "GeoLite2-ASN":
            print("download ASN")
            download_file("https://cdn.jsdelivr.net/npm/@ip-location-db/geolite2-asn-mmdb/geolite2-asn-ipv4.mmdb", args.dbdir+"GeoLite2-ASN.mmdb")

if __name__ == "__main__":
    main()